{% extends '!layout.html' %}

{%- block sidebarsearch %}
{{ super() }}
    <div id="searchbox">
	<h3>Google Search</h3>
    <form method="get" action="http://www.google.com/search">

    <table border="0" cellpadding="0">
    <tr><td>
    <input type="text"   name="q" size="20"
    maxlength="255" value="" />
    <input type="submit" value="Go" /></td></tr>
    <tr><td align="center" style="font-size:75%; color: #FFFFFF;">
    <input type="checkbox"  name="sitesearch"
    value="http://github.enthought.com/mayavi/mayavi/" checked /> only search Mayavi documentation<br />
    </td></tr></table>

    </form>
    </div>
    <hr/>
    <div id="searchbox">
	<a href="http://docs.enthought.com/mayavi/mayavi/misc.html#citing-mayavi"><h3>Citing Mayavi</h3></a>

	<p style="font-size:85%; color: #FFFFFF;">
    	If you publish articles using Mayavi, please 
	<strong><a href="misc.html#citing-mayavi">cite
	Mayavi</a></strong>. We need
	these citations to justify time and resources on the software.
	</p>
    </div>
{% endblock %}

{% block extrahead %}
<style type="text/css">
  body {
    /* Very bottom copyright, etc. info */
    background: #4d598d;
  }

  .documentwrapper {
    /* Blank area on left bar */
    background: #24326e;
  }

  .related {
    /* Long horizontal nav bars */
    background: #3c4b8a!important;
  }

  .sphinxsidebarwrapper {
    /* Filled part of left sidebar */
    background: #24326e;
  }

  a {
    color: #253370;
  }

  .sphinxsidebarwrapper a {
    color: #f2f2f2!important;
  }
</style>
{% endblock %}
